`timescale 1ns / 1ps
/*
 Floating Point Adder
 c = a + b

 floating point number
 - -------- -----------------------
 s   exp           significand
 1    8               23

 + exponent is shifted by 127
 + mantissa has 23 bits, 24 real bits (first bit is always 1)

 1. figure out the number with the smallest exponent
 2. shift that number so that the exponents are equal
 3. add significands

 */

module f_add(
	     a, //
	     b, //
	     c, //
	     op //
	     );

   input [31:0] a;
   input [31:0] b;
   input 	op;

   output [31:0] c;
   reg [31:0] 	 c;

   wire [7:0] 	 a_exp, b_exp;
   // reg [7:0] 	 a_exp, b_exp;
   reg [7:0] 	 c_exp;
   reg [23:0] 	 a_sig, b_sig;
   reg [24:0] 	 c_sig;
   reg 		 c_sgn;

   assign a_exp = a[30:23];
   assign b_exp = b[30:23];

   always @ (a or b) begin
      if (op == 1) begin // should complement b
      end

      a_sig = {1'b1, a[22:0]};
      b_sig = {1'b1, b[22:0]};

      if (a_exp < b_exp) begin
	 a_sig = a_sig >> (b_exp - a_exp);
	 c_exp = b_exp;
      end else begin
	 $display("Shifting...");
	 $display("%b", b_sig);
	 b_sig = b_sig >> (a_exp - b_exp);
	 $display("%d %d %b", a_exp, b_exp, b_sig >> (a_exp - b_exp));

	 c_exp = a_exp;
      end

      c_sgn = a_sig < b_sig ? b[31] : a[31];
      c_sig = a_sig + b_sig;
      $display("Sigs: %b %b %b\n", a_sig, b_sig, c_sig);

      while (c_sig != 0 && c_sig[24] == 0) begin
	 c_sig = c_sig << 1;
	 c_exp = c_exp - 1;
      end
      c_sig = c_sig << 1;

      c = {c_sgn, c_exp, c_sig[24:2]};
   end
endmodule // f_add

module f_add_tb;
   reg [31:0] a, b;
   wire [31:0] c;

   f_add add(a, b, c, 0);
   add add2(a, b, c, clk);
   

   initial begin
      $monitor("Exponents: %d %d %d\nSignificands: %b %b %b\n",
	       add.a_exp, add.b_exp, add.c_exp,
	       add.a_sig, add.b_sig, add.c_sig);

      // #100;
      // a = 32'h41400000;
      // b = 32'hc1100000;
      #100;
      a = 32'h41700000; // 15
      b = 32'h40400000; // 3
      #1000;

      $display("c = %x", add.c);
   end // initial begin

   always
     #5 clk = !clk;
endmodule

