module mem_unit(
		rtag,
		dc,

		rm,
		wm,
		busy,

		rram,
		wram,
		done,
		am,
		dm
		);
   inout [29:0] rtag;
   inout [31:0] dc;

   input 	rm;
   input 	wm;
   input 	busy;

   output 	rram;
   reg 		rram;
   output 	wram;
   reg 		wram;
   output 	done;
   reg 		done;
   output [29:0] am;
   reg [29:0] 	 am;
   output [31:0] dm;
   reg [31:0] 	 dm;

   reg [61439:0] mtag;
   reg [65535:0] cache;
   reg [22527:0] cu;

   wire [2047:0] busmtag;
   wire [2047:0] priout;
   wire [29:0] 	 adr_w;
   wire [29:0] 	 adr_r;
   wire [31:0] 	 data_r;
   wire [22527:0] cuout;
   wire [31:0] 	  ramout;

   wire [10:0] 	  pos_r;
   wire [10:0] 	  pos_w;

   asoc m_assoc(mtag, rtag, busmtag);
   prilib m_prilib(cu, priout);

   // priout = 1 pentru lru (scriere!)
   busfn_mtag m_busfn_mtag_write(mtag, priout, adr_w, pos_w);

   // busmtag = 1 (citire!)
   // busfn_mtag m_busfn_mtag_read(mtag, busmtag, adr_r, pos_r);
   busfn_cache m_busfn_cache_read(cache, busmtag, data_r, pos_r);

   dcz m_dcz(cu, cuout);
   ram m_ram(rram, wram, am, dm, ramout, busy);

   reg [31:0] 	  dc2;
   reg [5:0] 	  i;

   always @ (rm) begin
      cu = cuout; // decrementez contoare
      if (busmtag != 0) begin //  am valoarea in cache pe pos_r
	 for (i = 0; i < 11; i = i + 1)
	   cu[pos_r * 11 + i] = 1;
	 // rram == 0 => dc = data_r
      end else begin // nu am in cache, trebuie valoarea de la
	 // rtag din ram
	 rram = 1;
	 wram = 0;
	 am = rtag;
	 while (busy != 0) // astept valoarea in ramout
	   #5;
	 // o adaug la pozitia cea mai putin folosita
	 for (i = 0; i < 11; i = i + 1)
	   cu[pos_w * 11 + i] = 1;
	 for (i = 0; i < 30; i = i + 1)
	   mtag[pos_w * 30 + i] = rtag[i];
	 for (i = 0; i < 32; i = i + 1)
	   cache[pos_w * 32 + i] = ramout[i];
      end // else: !if(busmtag != 0)
      done = 1;
   end

   always @ (wm) begin
      for (i = 0; i < 31; i = i + 1)
	cache[pos_w * 32 + i] = dc[i];
      for (i = 0; i < 30; i = i + 1)
	   mtag[pos_w * 30 + i] = rtag[i];
      for (i = 0; i < 11; i = i + 1)
	cu[pos_w * 11 + i] = 1;
      rram = 0;
      wram = 1;
      am = rtag; // adresa de scris
      dm = dc; // valoarea de scris
      while (busy != 0)
	#5;
   end

   assign dc = rm == 1 ? (rram == 1 ? ramout : data_r) : dc;

endmodule
